@@ -2,7 +2,6 @@ import inspect
 import unittest
 
 import numpy as np
-import pkg_resources
 
 from Orange.classification import (
     LogisticRegressionLearner,
@@ -13,15 +12,18 @@ from Orange.classification import (
     ThresholdLearner,
 )
 from Orange.data import Table, Domain
-from Orange.regression import LinearRegressionLearner
+from Orange.regression import LinearRegressionLearner, CurveFitLearner
 from Orange.tests import test_regression, test_classification
 from Orange.widgets.data import owcolor
 from orangecontrib.explain.explainer import (
     compute_colors,
     compute_shap_values,
     explain_predictions,
+    INSTANCE_ORDERINGS,
+    get_instance_ordering,
     get_shap_values_and_colors,
     prepare_force_plot_data,
+    prepare_force_plot_data_multi_inst,
 )
 
 
@@ -92,7 +94,8 @@ class TestExplainer(unittest.TestCase):
     def test_kernel_explainer_sgd(self):
         learner = SGDClassificationLearner()
         model = learner(self.titanic)
-        np.random.shuffle(self.titanic.X)
+        with self.titanic.unlocked():
+            np.random.shuffle(self.titanic.X)
 
         shap_values, _, sample_mask, _ = compute_shap_values(
             model, self.titanic[:200], self.titanic[:200]
@@ -164,7 +167,7 @@ class TestExplainer(unittest.TestCase):
     def test_all_classifiers(self):
         """ Test explanation for all classifiers """
         for learner in test_classification.all_learners():
-            with self.subTest(learner.name):
+            with self.subTest(learner):
                 if learner == ThresholdLearner:
                     # ThresholdLearner require binary class
                     continue
@@ -181,15 +184,19 @@ class TestExplainer(unittest.TestCase):
                         self.iris.X.shape, shap_values[i].shape
                     )
 
-    @unittest.skipIf(
-        not hasattr(test_regression, "all_learners"),
-        "all_learners not available in Orange < 3.26",
-    )
     def test_all_regressors(self):
         """ Test explanation for all regressors """
         for learner in test_regression.all_learners():
-            with self.subTest(learner.name):
-                model = learner()(self.housing)
+            with self.subTest(learner):
+                if learner == CurveFitLearner:
+                    attr = self.housing.domain.attributes
+                    learner = CurveFitLearner(
+                        lambda x, a: np.sum(x[:, i] for i in range(len(attr))),
+                        [], [a.name for a in self.housing.domain.attributes]
+                    )
+                else:
+                    learner = learner()
+                model = learner(self.housing)
                 shap_values, _, _, _ = compute_shap_values(
                     model, self.housing, self.housing
                 )
@@ -304,7 +311,7 @@ class TestExplainer(unittest.TestCase):
     def test_explain_predictions(self):
         model = LogisticRegressionLearner()(self.iris)
 
-        shap_values, predictions, _, _ = explain_predictions(
+        shap_values, predictions, _, _, _ = explain_predictions(
             model, self.iris[:3], self.iris
         )
 
@@ -319,7 +326,7 @@ class TestExplainer(unittest.TestCase):
 
         # regression
         model = LinearRegressionLearner()(self.housing)
-        shap_values, predictions, _, _ = explain_predictions(
+        shap_values, predictions, _, _, _ = explain_predictions(
             model, self.housing[:3], self.housing
         )
 
@@ -329,6 +336,83 @@ class TestExplainer(unittest.TestCase):
         )
         self.assertTupleEqual((3, 1), predictions.shape)
 
+    def test_prepare_force_plot_data_multi_inst(self):
+        base_value = np.array([3])
+        shap_values = [
+            # 3 instances, 4 features
+            np.array([[1, -2, 6, 5], [-2, -3, -1, -5], [1, 2, 4, 5]])
+        ]
+        predictions = np.array([[1, 0, 0], [1, 0, 0], [1, 0, 0]])
+
+        idxs = get_instance_ordering(
+            shap_values[0], predictions[:, 0],
+            self.iris[:3], INSTANCE_ORDERINGS[2])
+
+        x_data, pos_data, neg_data = prepare_force_plot_data_multi_inst(
+            shap_values[0][idxs], base_value[0])
+
+        np.testing.assert_array_equal(x_data, np.arange(3))
+        self.assertEqual(len(pos_data), 4)
+        self.assertEqual(len(neg_data), 4)
+        self.assertEqual(len(pos_data[0]), 2)
+        self.assertEqual(len(neg_data[0]), 2)
+
+        for i in range(len(pos_data) - 1):
+            np.testing.assert_array_equal(pos_data[i][1], pos_data[i + 1][0])
+            np.testing.assert_array_equal(neg_data[i][1], neg_data[i + 1][0])
+
+        for i, (y1, y2) in enumerate([([13, -8, 15], [8, -8, 10]),
+                                      ([8, -8, 10], [2, -8,  6]),
+                                      ([2, -8,  6], [2, -8,  4]),
+                                      ([2, -8,  4], [1, -8,  3])]):
+            np.testing.assert_array_equal(pos_data[i][0], y1)
+            np.testing.assert_array_equal(pos_data[i][1], y2)
+
+        for i, (y1, y2) in enumerate([([13, -8, 15], [15, -5, 15]),
+                                      ([15, -5, 15], [15, 0, 15]),
+                                      ([15, 0, 15], [15,  2, 15]),
+                                      ([15, 2, 15], [15, 3, 15])]):
+            np.testing.assert_array_equal(neg_data[i][0], y1)
+            np.testing.assert_array_equal(neg_data[i][1], y2)
+
+    def test_prepare_force_plot_data_multi_inst_order(self):
+        base_value = np.array([3])
+        shap_values = [
+            np.array([[1, -2, 6, 5], [-2, -3, -1, -5], [1, 2, 4, 5]])
+        ]
+        predictions = np.array([[1, 0, 0], [1, 0, 0], [1, 0, 0]])
+
+        idxs = get_instance_ordering(
+            shap_values[0], predictions[:, 0],
+            self.iris[:3], self.iris.domain[0])
+
+        x_data, pos_data, neg_data = prepare_force_plot_data_multi_inst(
+            shap_values[0][idxs], base_value[0])
+
+        np.testing.assert_array_equal(x_data, np.arange(3))
+        self.assertEqual(len(pos_data), 4)
+        self.assertEqual(len(neg_data), 4)
+        self.assertEqual(len(pos_data[0]), 2)
+        self.assertEqual(len(neg_data[0]), 2)
+
+        for i in range(len(pos_data) - 1):
+            np.testing.assert_array_equal(pos_data[i][1], pos_data[i + 1][0])
+            np.testing.assert_array_equal(neg_data[i][1], neg_data[i + 1][0])
+
+        for i, (y1, y2) in enumerate([([15, -8, 13], [10, -8, 8]),
+                                      ([10, -8, 8], [6, -8, 2]),
+                                      ([6, -8, 2], [4, -8, 2]),
+                                      ([4, -8, 2], [3, -8, 1])]):
+            np.testing.assert_array_equal(pos_data[i][0], y1)
+            np.testing.assert_array_equal(pos_data[i][1], y2)
+
+        for i, (y1, y2) in enumerate([([15, -8, 13], [15, -5, 15]),
+                                      ([15, -5, 15], [15, 0, 15]),
+                                      ([15, 0, 15], [15,  2, 15]),
+                                      ([15, 2, 15], [15, 3, 15])]):
+            np.testing.assert_array_equal(neg_data[i][0], y1)
+            np.testing.assert_array_equal(neg_data[i][1], y2)
+
     def test_prepare_force_plot_data_target_0(self):
         shap_values = [
             np.array([[1, -2, 6, 5], [-2, -3, -1, -5], [1, 2, 4, 5]]),
@@ -576,17 +660,6 @@ class TestExplainer(unittest.TestCase):
         self.assertTupleEqual(self.iris.X.shape, shap_values[0].shape)
         self.assertTupleEqual((len(self.iris),), sample_mask.shape)
 
-    def test_remove_calibration_workaround(self):
-        """
-        When this test start to fail:
-        - remove the workaround in explainer.py-207:220 if allready fixed -
-        revert the pullrequest that adds those lines
-        - set minimum Orange version to 3.28.0
-        """
-        self.assertGreater(
-            "3.31.0", pkg_resources.get_distribution("orange3").version
-        )
-
 
 if __name__ == "__main__":
     unittest.main()
