@@ -13,6 +13,7 @@ from AnyQt.QtWidgets import QGraphicsItemGroup, QGraphicsWidget, \
 from Orange.base import Model
 from Orange.classification import RandomForestLearner
 from Orange.data import Table, Domain, ContinuousVariable, StringVariable
+from Orange.data.util import get_unique_names
 from Orange.regression import RandomForestRegressionLearner
 from Orange.widgets import gui
 from Orange.widgets.settings import Setting, ContextSetting, \
@@ -20,6 +21,7 @@ from Orange.widgets.settings import Setting, ContextSetting, \
 from Orange.widgets.utils.concurrent import TaskState
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.visualize.utils.customizableplot import Updater
+from Orange.widgets.widget import Output
 
 from orangecontrib.explain.explainer import \
     get_shap_values_and_colors, RGB_LOW, RGB_HIGH, temp_seed
@@ -113,9 +115,10 @@ class ViolinItem(FeatureItem):
     selection_changed = Signal(float, float, str)
 
     class SelectionRect(BaseSelectionRect):
-        def __init__(self, parent, width: int):
+        def __init__(self, parent, width: int, height: int):
             super().__init__(parent)
             self.parent_width = width
+            self.parent_height = height
 
     def __init__(self, parent, attr_name: str, x_range: Tuple[float],
                  width: int):
@@ -126,56 +129,40 @@ class ViolinItem(FeatureItem):
         parent.selection_cleared.connect(self.__remove_selection_rect)
 
     def set_data(self, x_data: np.ndarray, color_data: np.ndarray):
-        def put_point(_x, _y):
+        def place_point(_x, _y, _c):
             item = QGraphicsEllipseItem()
             item.setX(_x)
             item.setY(_y)
             item.setRect(0, 0, self.POINT_R, self.POINT_R)
-            color = QColor(*colors.pop().astype(int))
+            color = QColor(*_c)
             item.setPen(QPen(color))
             item.setBrush(QBrush(color))
             self._group.addToGroup(item)
 
         self._x_data = x_data
+        for x, y, c in zip(self._values_to_pixels(self._x_data),
+                           self._prepare_y_data(self._x_data),
+                           color_data):
+            place_point(x, y, c)
 
-        x_data_unique, dist, x_data = self.prepare_data()
-        for x, d in zip(x_data_unique, dist):
-            colors = color_data[x_data == np.round(x, 3)]
-            colors = list(colors[np.random.choice(len(colors), 11)])
-            y = self._height / 2 - self.POINT_R / 2
-            self.plot_data(put_point, x, y, d)
+    def _prepare_y_data(self, shaps: np.ndarray) -> np.ndarray:
+        with temp_seed(0):
+            n, nbins = len(shaps), 100
+            min_, max_ = np.min(shaps), np.max(shaps)
+            quant = np.round(nbins * (shaps - min_) / (max_ - min_ + 1e-8))
+            inds = np.argsort(quant + np.random.randn(n) * 1e-6)
 
-    def prepare_data(self) -> \
-            Optional[Tuple[np.ndarray, np.ndarray, np.ndarray]]:
-        x_data = self._values_to_pixels(self._x_data)
-        x_data = x_data[~np.isnan(x_data)]
-        if len(x_data) == 0:
-            return None
+        layer, last_bin = 0, -1
+        ys = np.zeros(n)
+        for ind in inds:
+            if quant[ind] != last_bin:
+                layer = 0
+            ys[ind] = np.ceil(layer / 2) * ((layer % 2) * 2 - 1)
+            layer += 1
+            last_bin = quant[ind]
 
-        x_data = np.round(x_data - self.POINT_R / 2, 3)
-
-        # remove duplicates and get counts (distribution) to set y
-        x_data_unique, counts = np.unique(x_data, return_counts=True)
-        min_count, max_count = np.min(counts), np.max(counts)
-        dist = (counts - min_count) / (max_count - min_count)
-        if min_count == max_count:
-            dist[:] = 1
-        dist = dist ** 0.7
-
-        # plot rarest values first
-        indices = np.argsort(counts)
-        return x_data_unique[indices], dist[indices], x_data
-
-    @staticmethod
-    def plot_data(func: Callable, x: float, y: float, d: float):
-        func(x, y)  # y = 0
-        if d > 0:
-            offset = d * 10
-            func(x, y + offset)  # y = (0, 10]
-            func(x, y - offset)  # y = (-10, 0]
-            for i in range(2, int(offset), 2):
-                func(x, y + i)  # y = [2, 8]
-                func(x, y - i)  # y = [-8, -2]
+        ys = ys * 0.4 / np.max(ys + 1)
+        return self._height / 2 - self.POINT_R / 2 + ys * self._height
 
     def _values_to_pixels(self, x: np.ndarray) -> np.ndarray:
         # scale data to [-0.5, 0.5]
@@ -183,7 +170,7 @@ class ViolinItem(FeatureItem):
         # round data to 3. decimal for sampling
         x = np.round(x, 3)
         # convert to pixels
-        return x * self._width + self._width / 2
+        return x * self._width + self._width / 2 - self.POINT_R / 2
 
     def _values_from_pixels(self, p: np.ndarray) -> np.ndarray:
         # convert from pixels
@@ -192,17 +179,12 @@ class ViolinItem(FeatureItem):
         return np.round(x * self._range / self.SCALE_FACTOR, 3)
 
     def rescale(self, width):
-        def move_point(_x, *_):
-            item = next(points)
-            item.setX(_x)
-
         self._width = width
         self.updateGeometry()
-        points = (item for item in self._group.childItems())
 
-        x_data_unique, dist, _ = self.prepare_data()
-        for x, d in zip(x_data_unique, dist):
-            self.plot_data(move_point, x, 0, d)
+        for x, item in zip(self._values_to_pixels(self._x_data),
+                           self._group.childItems()):
+            item.setX(x)
 
         if self._selection_rect is not None:
             old_width = self._selection_rect.parent_width
@@ -215,7 +197,20 @@ class ViolinItem(FeatureItem):
 
     def set_height(self, height: float):
         self._height = height + self.HEIGHT
-        # TODO
+        for y, item in zip(self._prepare_y_data(self._x_data),
+                           self._group.childItems()):
+            item.setY(y)
+
+        if self._selection_rect is not None:
+            old_height = self._selection_rect.parent_height
+            rect = self._selection_rect.rect()
+            y1 = self._height * rect.y() / old_height
+            y2 = self._height * (rect.y() + rect.height()) / old_height
+            rect = QRectF(rect.x(), y1, rect.width(), y2 - y1)
+            self._selection_rect.setRect(rect)
+            self._selection_rect.parent_height = self._height
+
+        self.updateGeometry()
 
     def __remove_selection_rect(self):
         if self._selection_rect is not None:
@@ -227,7 +222,8 @@ class ViolinItem(FeatureItem):
     def add_selection_rect(self, x1, x2):
         x1, x2 = self._values_to_pixels(np.array([x1, x2]))
         rect = QRectF(x1, 0, x2 - x1, self._height)
-        self._selection_rect = ViolinItem.SelectionRect(self, self._width)
+        self._selection_rect = ViolinItem.SelectionRect(
+            self, self._width, self._height)
         self._selection_rect.setRect(rect)
 
     def mousePressEvent(self, event: QGraphicsSceneMouseEvent):
@@ -237,7 +233,7 @@ class ViolinItem(FeatureItem):
         if event.buttons() & Qt.LeftButton:
             if self._selection_rect is None:
                 self._selection_rect = ViolinItem.SelectionRect(
-                    self, self._width)
+                    self, self._width, self._height)
             x = event.buttonDownPos(Qt.LeftButton).x()
             rect = QRectF(x, 0, event.pos().x() - x, self._height).normalized()
             rect = rect.intersected(self.contentsRect())
@@ -346,6 +342,9 @@ class OWExplainModel(OWExplainFeatureBase):
         "orangecontrib.prototypes.widgets.owexplainmodel.OWExplainModel"
     ]
 
+    class Outputs(OWExplainFeatureBase.Outputs):
+        impact = Output("Impact", Table)
+
     settingsHandler = ClassValuesContextHandler()
     target_index = ContextSetting(0)
     show_legend = Setting(True)
@@ -365,16 +364,13 @@ class OWExplainModel(OWExplainFeatureBase):
             contentsLength=12)
 
         super()._add_controls()
-
-        # TODO - implement zoom - ViolinItem.set_height()
-        self.controls.zoom_level.parent().setVisible(False)
-
         gui.checkBox(self.display_box, self, "show_legend", "Show legend",
                      callback=self.__show_check_changed)
 
     def __target_combo_changed(self):
         self.update_scene()
         self.update_scores()
+        self.update_impact()
         self._clear_selection()
 
     def __show_check_changed(self):
@@ -461,6 +457,30 @@ class OWExplainModel(OWExplainFeatureBase):
         scores_table.name = "Feature Scores"
         return scores_table
 
+    def update_impact(self):
+        impact = None
+        if self.results is not None:
+            impact = self.get_impact_table()
+        self.Outputs.impact.send(impact)
+
+    def get_impact_table(self) -> Table:
+        data = self.data
+        x = self.results.x[self.target_index]
+        mask = self.results.mask
+        proposed = [f"I({n})" for n in self.results.names]
+        names = [v.name for v in data.domain.class_vars + data.domain.metas]
+        proposed = get_unique_names(names, proposed)
+        domain = Domain([ContinuousVariable(n) for n in proposed],
+                        data.domain.class_vars, metas=data.domain.metas)
+        impact_table = Table(domain, x, data.Y[mask], data.metas[mask])
+        impact_table.name = "Feature Impact"
+        return impact_table
+
+    # Concurrent
+    def on_done(self, results: Optional[BaseResults]):
+        super().on_done(results)
+        self.update_impact()
+
     # Misc
     def send_report(self):
         if not self.data or not self.model:
